# Launchable's ingester Docker image with tag
# If you need up-to-date image, please replace the image tag
# You can see the docker image tags at
# https://hub.docker.com/repository/registry-1.docker.io/launchableinc/ingester/tags
ingester_image = (
    'launchableinc/ingester:'
    '976171f4005f8a1c6d0e706d20760890b44c5373'
)
