/*
 * This Java source file was generated by the Gradle 'init' task.
 */
package com.launchableinc.rocket_car_gradle;

import org.junit.Test;
import static org.junit.Assert.*;

public class App2Test {
    @Test public void testAppHasAGreeting() {
        App classUnderTest = new App();
        assertNotNull("app should have a greeting", classUnderTest.getGreeting());
    }

    @Test public void testAppHasAGreeting2() {
        App classUnderTest = new App();
        assertNotNull("app should have a greeting", classUnderTest.getGreeting());
    }

}
